from pandas.core.frame import DataFrame


class MergedKG:
    def __init__(self,
                 nodes: DataFrame,
                 edges: DataFrame,
                 duplicate_nodes: DataFrame,
                 dangling_edges: DataFrame
                 ):
        self.nodes = nodes
        self.edges = edges
        self.duplicate_nodes = duplicate_nodes
        self.dangling_edges = dangling_edges



import os, tarfile
from pathlib import Path
import pandas as pd
from typing import List, Tuple

from cat_merge.model.merged_kg import MergedKG

def get_files(filepath: str):
    node_files = []
    edge_files = []
    print(filepath)
    for file in os.listdir(filepath):
        if file.endswith('nodes.tsv'):
            node_files.append(file)
        elif file.endswith('edges.tsv'):
            edge_files.append(file)
    return node_files, edge_files

def read_dfs(files: List[str], add_provided_by: bool = True) -> List[pd.DataFrame]:
    dataframes = []
    for file in files:
        df = pd.read_csv(file, sep="\t", dtype="string", lineterminator="\n", index_col='id')
        df.index.name = 'id'
        if add_provided_by:
            df["provided_by"] = os.path.basename(file)
        dataframes.append(df)
    return dataframes

def write_df(df: pd.DataFrame, filename: str):
    df.to_csv(filename, sep="\t")


def write_tar(tar_path: str, files: List[str], delete_files=True):
    tar = tarfile.open(tar_path, "w:gz")
    for file in files:
        tar.add(file, arcname=os.path.basename(file))
    tar.close()
    if delete_files:
        for file in files:
            os.remove(file)


def write(kg: MergedKG, name: str, output_dir: str):

    os.makedirs(output_dir, exist_ok=True)

    duplicate_nodes_path = f"{output_dir}/qc/{name}-duplicate-nodes.tsv.gz"
    dangling_edges_path = f"{output_dir}/qc/{name}-dangling-edges.tsv.gz"
    nodes_path = f"{output_dir}/{name}_nodes.tsv"
    edges_path = f"{output_dir}/{name}_edges.tsv"
    tar_path = f"{output_dir}/{name}.tar.gz"

    write_df(df=kg.duplicate_nodes, filename=duplicate_nodes_path)
    write_df(df=kg.dangling_edges, filename=dangling_edges_path)
    write_df(df=kg.nodes, filename=nodes_path)
    write_df(df=kg.edges, filename=edges_path)

    write_tar(tar_path, [nodes_path, edges_path])



from typing import List
from cat_merge.file_utils import *
from cat_merge.merge_utils import *

def merge(
    name: str = "merged-kg",#typer.Option("merged-kg", help="Name of the resulting knowledge graph"),
    input_dir: str = None,#typer.Option(None, help="Optional directory containing node and edge files"),
    edges: List[str] = None,#typer.Option(None, help="Optional list of edge files"),
    nodes: List[str] = None,#typer.Option(None, help="Optional list of node files"),
    output_dir: str = "merged-output",#typer.Option("merge-output", help="Directory to output knowledge graph")
    ):

    print(f"Name: {name} // input_dir: {input_dir} // nodes: {nodes} // edges: {edges} // output_dir: {output_dir}")

    if nodes is not None and edges is not None:
        node_dfs = read_dfs(nodes)
        edge_dfs = read_dfs(edges)
    elif input_dir is not None:
        node_files, edge_files = get_files(input_dir)
        node_dfs = read_dfs(node_files)
        edge_dfs = read_dfs(edge_files)

    write(
        name=name,
        kg=merge_kg(node_dfs=node_dfs, edge_dfs=edge_dfs),
        output_dir=output_dir
    )



import pandas as pd
from pandas.core.frame import DataFrame
from typing import List
import os
from cat_merge.model.merged_kg import MergedKG


def concat_dataframes(dataframes: List[DataFrame]) -> DataFrame:
    return pd.concat(dataframes, axis=0)


def get_duplicate_rows(df: DataFrame) -> DataFrame:
    return df[df.index.duplicated(keep=False)]


def clean_nodes(nodes: DataFrame) -> DataFrame:
    nodes.reset_index(inplace=True)
    nodes.drop_duplicates(inplace=True)
    nodes = nodes.rename(columns={'index': 'id'})
    nodes.fillna("None", inplace=True)
    column_agg = {x: ' '.join for x in nodes.columns if x != 'id'}
    nodes = nodes.groupby(['id'], as_index=True).agg(column_agg)
    return nodes


def clean_edges(edges: DataFrame, nodes: DataFrame) -> DataFrame:
    return edges[edges.subject.isin(nodes.index) & edges.object.isin(nodes.index)]


def get_dangling_edges(edges: DataFrame, nodes: DataFrame) -> DataFrame:
    return edges[~edges.subject.isin(nodes.index) | ~edges.object.isin(nodes.index)]


def merge_kg(edge_dfs: List[DataFrame], node_dfs: List[DataFrame]) -> MergedKG:

    all_nodes = concat_dataframes(node_dfs)
    all_edges = concat_dataframes(edge_dfs)

    duplicate_nodes = get_duplicate_rows(df=all_nodes)
    dangling_edges = get_dangling_edges(edges=all_edges, nodes=all_nodes)

    nodes = clean_nodes(nodes=all_nodes)
    edges = clean_edges(edges=all_edges, nodes=nodes)

    return MergedKG(nodes=nodes, edges=edges, duplicate_nodes=duplicate_nodes, dangling_edges=dangling_edges)
